;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xda414578f29222291e1ff18a7b254940 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x106114af757664de6ec7f214b0f1c2ed . "./code.cg2js/cg-gesture-emulation.fasl")
(#x3c090fd286018cf8d9391a72f7a34f84 . "./code.cg2js/cg-grid-widget.fasl")
(#x6790f8a2d087d3ca0ce131a12562866b . "./code.cg2js/cg-utility-dialog.fasl")
(#x0d2aab67c11c16577e8614966e4136d9 . "./code.cg2js/cg-wrap-string.fasl")
(#x60a2368bca45cb449bda15413d08dacd . "./code.cg2js/cg-editable-text.fasl")
(#xa4e1507d9f6d1c33bfe5da1037acff9a . "./code.cg2js/cg-base.fasl")
(#x0fec74e8544308d3bf71b5c1610a4003 . "./code.cg2js/cg-lisp-text.fasl")
(#xd4373018d894a60463ea957ce5875b3b . "./code.cg2js/cg-intersect.fasl")
(#x698b5be80dc85b2df8de4cea01be9a3f . "./code.cg2js/cg-choose-printer.fasl")
(#xb0b34757cb1ae9de938fa6b2211bc52a . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x16a5987b3b5364e035fb6d94d4d11b5c . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xc8e78ff52afd52a9f41b5b07d777d844 . "./code.cg2js/cg-font-dialog.fasl")
(#x8c9675c3bb1c284590796ecdc0f83b12 . "./code.cg2js/cg-comtab.fasl")
(#xbaa02b36cec33551206637be0a57689a . "./code.cg2js/cg-tooltip.fasl")
(#x3e68f62ccb401fbdbea3be108b8e88fc . "./code.cg2js/cg-chart-or-plot.fasl")
(#x528df25af2f8a87317c5237dc840b54f . "./code.cg2js/cg-get-pixmap.fasl")
(#x73d1ec68e20d480567a87bed38914bc9 . "./code.cg2js/cg-status-bar.fasl")
(#x1adcd9926f4e2e01584b0874b51cb311 . "./code.cg2js/cg-tab-control.fasl")
(#x0f9ffe52bf15cb5107ab2af6e11a9479 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x5e4e6198aa8c5595555130640a775622 . "./code.cg2js/cg-list-view.fasl")
(#xd37c480327ad00779698900b371c32bb . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x6bd447769862f53981f6c9a274343211 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x2b22897acef19ed2f55f8134c694f7ad . "./code.cg2js/cg-project-window.fasl")
(#x48b027679bfdfae5a5d4ebdadff244db . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x65153d6c34121b011b10f092d74c76e1 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x9eb927bd9f1be5af3c1fe959b8d26588 . "./code.cg2js/cg-scaling-stream.fasl")
(#xce800e1cc46177e6a032e5118730110d . "./code.cg2js/cg-bitmap-stream.fasl")
(#x1a900873ffe54d1f1128d0e6f3e0f336 . "./code.cg2js/cg-menu.fasl")
(#x057565f3c0897121326dd14d1165b07c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x79d9fe6b6db4ed59ecf87f728542ed5c . "./code.cg2js/cg-intersect-posbox.fasl")
(#xf492c6e43b6d4f359287eac7949a9dcf . "./code.cg2js/cg-split-bar.fasl")
(#x3dd135e1b0e8d3fa3ca0202943f4fd83 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x69383329ed06158eb621ca2d73da01f1 . "./code.cg2js/cg-timer.fasl")
(#xcec987fd90e7918a177cec42f1139016 . "./code.cg2js/cg-text-or-combo.fasl")
(#x57e4ec95bfc90ede44ba02aa5aa3f25d . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x2d775f3fa4937e4ae65cb2702916861b . "./code.cg2js/cg-string-dialog.fasl")
(#xf47d02a4317ec02c235e293aadde3be3 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x389b4e2551030de74e3092be5038949a . "./code.cg2js/cg-menu-tooltip.fasl")
(#xbb615125906284f27f54c6f14dbdae23 . "./code.cg2js/cg-outline.fasl")
(#x78b3892180da1fa81ed2e7865a70c8bc . "./code.cg2js/cg-lettered-menu.fasl")
(#x4182e137f760cba30211032a02e304a9 . "./code.cg2js/cg-graphics-context.fasl")
(#x714801593d379f200343aafe6a4960c6 . "./code.cg2js/cg-item-list.fasl")
(#xdd8513273c85def24edbe42dcbd8ec99 . "./code.cg2js/cg-caret.fasl")
(#xd1908e6533bf0cb74e3b3005ee0eacb1 . "./code.cg2js/cg-printing.fasl")
(#x81aae5a66c52a597b62d88c3fdd0c270 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xa873ace757b7c1664399a9e835fab7c8 . "./code.cg2js/cg-progress-indicator.fasl")
(#x6936b72a9d269da7da56f7927dfab5ae . "./code.cg2js/cg-dde-utils.fasl")
(#x56a171bd89f681a267882d00a657ced1 . "./code.cg2js/cg-plot-widget.fasl")
(#x98706f7d9588ac2670d72d2b8fc3fd8e . "./code.cg2js/cg-os-window.fasl")
(#x58ab21df4ba9f88808fc355401267b44 . "./code.cg2js/cg-calendar.fasl")
(#x0de4af2b279225447e617aa845861889 . "./code.cg2js/cg-chart-widget.fasl")
(#x06bb51530ccf3cc8fae7b355294e6c4d . "./code.cg2js/cg-file-dialog.fasl")
(#x4ceb19c2c0cd1ed3e46bbd3c9f544443 . "./code.cg2js/cg-nodes.fasl")
(#xd40dc0f3b1283a5b903fdcca4bd00f52 . "./code.cg2js/cg-lamp.fasl")
(#xbcbc0ca7901a35377884cc742175d0f4 . "./code.cg2js/cg-html-dialog.fasl")
(#x859d75f538de11e594998b628b2cd70b . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xcd88b04a65e42e2e166cc515faaf8cfe . "./code.cg2js/cg-class-slot-grid.fasl")
(#xc019878f1161e561be6c8f2f4e05f4c7 . "./code.cg2js/cg-choice-list.fasl")
(#x93403caa230bc0db4d867d950d950ad4 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xff42ee189ffff41351686986ca1ab1f9 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x30ef687bdb56c86bc670df94089a11cd . "./code.cg2js/cg-hotspot.fasl")
(#x686f7de28599285ea0836a92587e5ff2 . "./code.cg2js/cg-property.fasl")
(#xc7a3fbabfad1228ae05bee0c15eb8159 . "./code.cg2js/cg-dropping-outline.fasl")
(#x269659dd61372cc93a0fbd45bb71a708 . "./code.cg2js/cg-template-string.fasl")
(#xc6b4f7ced223e1b40a65429b02780be5 . "./code.cg2js/cg-palette.fasl")
(#x218f66ad6480a4389cd53ba54b3acc45 . "./code.cg2js/cg-selected-object.fasl")
(#xd330530137f88c4662d8768a12113d01 . "./code.cg2js/cg-up-down-control.fasl")
(#x9e0cba59a2fb540c06baa3ea9040c321 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x8f5953d726c450d7b85b2ac42329aa01 . "./code.cg2js/cg-layout.fasl")
(#x21c60317e6f92c1fdcadd867d794c5c4 . "./code.cg2js/cg-get-position.fasl")
(#x6759118a7a4774d9ea9ff3198a2161c7 . "./code.cg2js/cg-lisp-widget.fasl")
(#x0a77a281ad00370d98197e47c608ea6a . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x1736f1d34c9a332115bd256b8994287a . "./code.cg2js/cg-object-editor.fasl")
(#xe1013deddbda276e004fdcde6dd88b62 . "./code.cg2js/cg-class-grid.fasl")
(#x890853f444b8ddad7cfb7a8c27768b91 . "./code.cg2js/cg-yes-no-string.fasl")
(#x8fa2c30e820cf4faef8a9f8209c2b041 . "./code.cg2js/cg-toggling-widget.fasl")
(#x37f2d8d71cd6a24d223f3fb691fe144d . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xc3375b5f500aa3b3a47f9c6449cbd46c . "./code.cg2js/cg-button.fasl")
(#x184248f8b2faddc56f6d79a871947ed6 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x5554f185a6d7286c867f587b1ebd1a2e . "./code.cg2js/cg-scroll-bar.fasl")
(#x10af78b673d50ded25984ae40ec2d183 . "./code.cg2js/cg-combo-box.fasl")
(#xa9d0b2e96e4143a7e48253e269503c04 . "./code.cg2js/cg-web-browser.fasl")
(#x0a32f8cf8d4be20cb41f248cd4d87beb . "./code.cg2js/cg-drawable.fasl")
(#xdb938df59dbbb998adcf83fa279cbaf7 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x89673677666050c3f9bf488d5836dea4 . "./code.cg2js/cg-message-dialog.fasl")
(#x2d310c02c6774b99702b54243f368ed1 . "./code.cg2js/cg-text-widget.fasl")
(#x2ff46a0ed2506ca906ae18fc6ee111f6 . "./code.cg2js/cg-static-text.fasl")
(#xfd61bfdc9261c3bc1c5727b1aedb4bd6 . "./code.cg2js/cg-class-support.fasl")
(#x84334d5f0944ac6c56d8048b1af577e2 . "./code.cg2js/cg.fasl")
(#x928b571717fcf55098f65f442c62a71b . "./code.cg2js/cg-directory-dialog.fasl")
(#xa7d39285221294e527b9dcdac6ef647b . "./code.cg2js/cg-clipboard.fasl")
(#xeb7fa24fa49126f4fb03e9d12784c9ea . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xce263ee827f835ab5d27718aeb9da04c . "./code.cg2js/cg-bitmap-pane.fasl")
(#x91302dc1f06362977f068b5625b4353c . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x75bd4c3db9c652b37174844fb5fd16ef . "./code.cg2js/cg-check-box.fasl")
(#xb8d6d1fcf60782542819dfef1329b732 . "./code.cg2js/cg-color-dialog.fasl")
(#x12b14a0c96fcec567ca52e6f6c15e7c1 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x112a332c5d13db4ed22b68087fb76ab8 . "./code.cg2js/cg-dialog-item.fasl")
(#x4715c66fff8b6711398959e79077396b . "./code.cg2js/cg-edit-in-place.fasl")
(#xe79f961ea79701e8758b9487dde64b25 . "./code.cg2js/cg-tray.fasl")
(#xf6795c2cde8e7a46e981873fe09c9793 . "./code.cg2js/cg-paren-matching.fasl")
(#xca10eb3bc20c0609cc7da71c794a871d . "./code.cg2js/cg-player.fasl")
(#xf4facdf7daf8ee927121f7f759812938 . "./code.cg2js/cg-yes-no-list.fasl")
(#xd9bc35e1a43d72aa47c75234b4450467 . "./code.cg2js/cg-common-control.fasl")
(#x8458ef79bcf28da810db3d81245c225d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xc6db154aa1f0e13ebdc708d22d0a71bf . "./code.cg2js/cg-os-widget.fasl")
(#x24a4f3f010e4b3aaabb0ca332984ca0b . "./code.cg2js/cg-sample-file-menu.fasl")
(#x21802f56ecfedbcdc204b4fdd3113f40 . "./code.cg2js/cg-trackbar.fasl")
(#x89a8dd35cbc2a8af01614089a0c1fafe . "./code.cg2js/cg-find-string-dialog.fasl")
(#xac713c62f10350b40ff9c6ba4ad0db8e . "./code.cg2js/cg-html-widget.fasl")
(#x8a2650b8044150810de58414a9debe21 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xcd049085ae34f0d1956dfe57d44627a1 . "./code.cg2js/cg-acache.fasl")
(#xfe9420f8ca72d71b5ecd8816feff0b1b . "./code.cg2js/cg-shortcut-menu.fasl")
(#xfc7e168a89809af17a69048c2a6c9939 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x97dcba29ffeeeebe455cd130c9c669a5 . "./code.cg2js/cg-icon.fasl")
(#x565ded8065f5bdec6342a9f8da4a5e84 . "./code.cg2js/cg-pixmap.fasl")
(#xdcfb3a9fb6522818b91929beaca44d41 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x8306d0b30fdd1edaadaa77e1249d9209 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x379dd370ded9086665a6a3aa7813b45a . "./code.cg2js/cg-toolbar.fasl")
(#x0fcf554d3c12d31a7916fd6778bf3cc5 . "./code.cg2js/cg-radio-button.fasl")
(#xe72c08c9f398fe06e39016e0a3ed4d65 . "./code.cg2js/cg-curve.fasl")
(#xff63c774e78be3ce522c5f0ebfcdf3af . "./code.cg2js/cg-picture-widget.fasl")
(#xa2ca63a5ecf5266be336b55623e99aa7 . "./code.cg2js/cg-dev.fasl")
(#x4c0f96e5ec1c27e9df0ca2d510825dff . "./code.cg2js/cg-group-box.fasl")
(#xa7a9b713fded99d31bc2ba2dbe967b6c . "./code.cg2js/cg-fill-texture.fasl")
(#x19611441d7f44e437128a96d1ed5b6ac . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x6a3b695add7808f201dbe5a733d65617 . "./code.idejs/ide-keyshort.fasl")
(#xb8adcf74a64e69841c68e2352e60329f . "./code.idejs/ide-form.fasl")
(#xa28c17da759e3bafddf6ecd49ea90812 . "./code.idejs/ide-find-in-files.fasl")
(#xe35f75943469b0c905ac1ad998807a09 . "./code.idejs/ide-class-grapher.fasl")
(#xe33b45ec04731494e28aee082025d93a . "./code.idejs/ide-base.fasl")
(#xf3ffcb83ab44e0bd79d188d8e4bdb559 . "./code.idejs/ide-class-browser.fasl")
(#xd6a16217a74d27c3db3fb0c65b0a5821 . "./code.idejs/ide-find-definitions.fasl")
(#x132cdbfeae6d505a14b6b715f4506a14 . "./code.idejs/ide-package-list.fasl")
(#xa8c69667027e19879f55ca967a42a466 . "./code.idejs/ide-grapher.fasl")
(#x272e7c0e28a19459beff843a2646daae . "./code.idejs/ide-dev.fasl")
(#xce1b98b6913e56af020f8de99d7dcdb9 . "./code.idejs/ide.fasl")
(#x1c0cac1f233e91941a45447ada0c56e7 . "./code.idejs/ide-trace.fasl")
(#x7693452a6d7cfedaeb3c15f074f619d2 . "./code.idejs/ide-apropos.fasl")
(#xc0424edd2e2a01ff5b2fd59e0f388812 . "./code.idejs/ide-inspector.fasl")
(#x6be1fef990798e34f76ebbd23ba5a494 . "./code.idejs/ide-acache.fasl")
(#x8074b5fab6956233e69c7c7e2f969e83 . "./code.idejs/ide-project.fasl")
(#xc1c63a66ce01e5c8dbff3f120edc8183 . "./code.idejs/ide-window-list.fasl")
(#xd0f21ce545502da9d2ed4359d1789626 . "./code.idejs/ide-intro.fasl")
(#x10f15f281c58ac3e8e01c26d0d27fd02 . "./code.idejs/ide-clipboard.fasl")
(#x3d9cf4a010775419998db7aff14d1d4b . "./code.idejs/ide-editor.fasl")
(#xf9d7d504d04852c9fea174c62047f335 . "./code.idejs/ide-process-browser.fasl")
(#x4ef1fd479580ae296dbbcd75827ed6f8 . "./code.idejs/ide-patch-dialog.fasl")
(#x407f0b05d85553d6ed47a73eee61af4f . "./code.idejs/ide-navigator.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
